"""
        N. H. Timm (1975).
          Multivariate analysis with applications in
                education and psychology.
             Brooks/Cole publishing company

        岡本安晴（2014）心理学データ分析と測定．
        　　第３章　偏回帰係数を比べる：コントラスト
        　勁草書房　　　　　
"""
import numpy as np
import scipy.stats as ss
import sys

infilenm = input('入力データファイル名(*.txt) = ')  
with open(infilenm, 'r', encoding = 'utf-8') as fin:  #   日本語はUTF-8で設定
    raw_data = fin.readlines()

fout = open('Results.txt', 'w')
fout.write(f'入力データファイル = {infilenm}\n')

pre_data = ''
for v in raw_data:
    pre_data += v.replace('\t', ' ').replace('\n', ',')
pre1_data = pre_data.split(',')
ck = 0
pos = 0
while True:
    if len(pre1_data[pos]) > 0:
        if pre1_data[pos][0] == '/':
            pos += 1
            break
    pos += 1
p_q = np.array(pre1_data[pos].split(), dtype='int')
p = p_q[0]                 #   従属変数の個数
q = p_q[1]                 #   独立変数の個数
print('p =',p, '  q =',q)
#
#           データY,Xの読み込み
#
pos += 1
preX = []
while True:
    if pre1_data[pos][0] == '/':
        pos += 1
        break
    v = np.array(pre1_data[pos].split(), dtype='float')
    print(v)
    preX.append(v)
    pos += 1
preX = np.array(preX)
#print('preX =\n', preX)

Y = preX[:,0:p]
X = preX[:,p:p+q]
r = np.linalg.matrix_rank(X)
N = len(X)
if p > N-r:
    print(f'\nthat p <= N-r should hold.\np = {p},  N = {N},  r = {r}\n')
    sys.exit()

fout.write('\n\n')
for i in range(p):
    vs = f'Y{i+1}'
    fout.write(f'{vs:>10s}')
for i in range(q):
    vs = f'X{i+1}'
    fout.write(f'{vs:>10s}')
fout.write('\n')
for i in range(N):
    for j in range(p):
        vs = f'{Y[i][j]}'
        fout.write(f'{vs:>10s}')
    for j in range(q):
        vs = f'{X[i][j]}'
        fout.write(f'{vs:>10s}')
    fout.write('\n')
    
ginvXX = np.linalg.pinv(X.T @ X)
H = ginvXX @ X.T @ X
Bhat = ginvXX @ X.T @ Y
print('Bhat =\n', Bhat)
#
#            コントラストの検定
#
while True:
    while True:
        if len(pre1_data[pos]) > 0:
               if pre1_data[pos][0] == '/':
                   pos += 1
                   break
        pos += 1
    g = int(pre1_data[pos])
    if g < 1:                 #   検定の終了
        break
    if g > r:
        print(f'g(={g}) should be smaller or equal to f(={r})!\n')
        continue
    pos += 1
    #
    #               Cの読み込み
    #
    C = []
    for i in range(g):
        C.append(pre1_data[pos].split())
        pos += 1
    C = np.array(C, dtype='float')
    print('\n\nC =\n', C)
    fout.write('\n\n')
    fout.write('\nC =\n')
    fout.write(C.__str__())
    if np.trace((C - C@H) @ (C - C@H).T) > 1.0e-9:
        print('C is not estimable\nC =\n', C)
        continue
    while True:
        if len(pre1_data[pos]) > 0:
               if pre1_data[pos][0] == '/':
                   pos += 1
                   break
        pos += 1
        
    u = int(pre1_data[pos])
    pos += 1
    if u > p:
        print(f'\nu(={u}) should be smaller or eual to p(={p})\n')
    #
    #        Aの読み込み
    #
    A = []
    for i in range(p):
        A.append(pre1_data[pos].split())
        pos += 1
    A = np.array(A, dtype='float')
    print('A =\n', A)
    fout.write('\n\nA =\n')
    fout.write(A.__str__())
    #
    #      岡本安晴（2014）、pp.45-47.
    #
    preQea = (Y - X @ Bhat) @ A
    Qea = preQea.T @ preQea
    Qta = Qea + (C @ Bhat @ A).T @ np.linalg.inv(C @ ginvXX @ C.T) @\
          (C @ Bhat @ A)
    Lmbd = np.linalg.det(Qea) / np.linalg.det(Qta)
    print('\nLambda =', Lmbd, ' df1 =', u, ' df2 =', g, ' df3 =', N-r)
    fout.write('\n\n')
    fout.write(f'Lambda = {Lmbd:.5f},   df1 = {u},  df2 = {g},  df3 = {N-r}')
    if u == 1:
        F = ((1.0 - Lmbd)/g) / ((Lmbd/(N-r)))
        print(f'\nF = {F:.5f},   df1 = {g}, df2 = {N-r},', end ='')
        cdf = ss.f.cdf(F, g, N-r)
        print(f'     p = {1-cdf:.5f}')
        fout.write('\n\n')
        fout.write(f'F = {F:.5f},   df1 = {g},  df2 = {N-r},')
        fout.write(f'  p = {1-cdf:.5f}')

    chi2 = -((N-r) -0.5*(u-g+1)) * np.log(Lmbd)
    print(f'\nChi~2 = {chi2:.5f},   df = {u*g},', end = '')
    cdf = ss.chi2.cdf(chi2, u*g)
    print(f'    p = {1-cdf:.5f}')
    fout.write('\n\n')
    fout.write(f'Chi^2 = {chi2:.5f},   df = {u*g},   p = {1-cdf:.5f}')

fout.close()
print('\nResults.txt was saved.')
    
               
               
